# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from lndmanage.grpc_compiled import lightning_pb2 as lightning__pb2


class LightningStub(object):
    """
    Comments in this file will be directly parsed into the API
    Documentation as descriptions of the associated method, message, or field.
    These descriptions should go right above the definition of the object, and
    can be in either block or // comment format.

    An RPC method can be matched to an lncli command by placing a line in the
    beginning of the description in exactly the following format:
    lncli: `methodname`

    Failure to specify the exact name of the command will cause documentation
    generation to fail.

    More information on how exactly the gRPC documentation is generated from
    this proto file can be found here:
    https://github.com/lightninglabs/lightning-api

    Lightning is the main RPC server of the daemon.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.WalletBalance = channel.unary_unary(
                '/lnrpc.Lightning/WalletBalance',
                request_serializer=lightning__pb2.WalletBalanceRequest.SerializeToString,
                response_deserializer=lightning__pb2.WalletBalanceResponse.FromString,
                )
        self.ChannelBalance = channel.unary_unary(
                '/lnrpc.Lightning/ChannelBalance',
                request_serializer=lightning__pb2.ChannelBalanceRequest.SerializeToString,
                response_deserializer=lightning__pb2.ChannelBalanceResponse.FromString,
                )
        self.GetTransactions = channel.unary_unary(
                '/lnrpc.Lightning/GetTransactions',
                request_serializer=lightning__pb2.GetTransactionsRequest.SerializeToString,
                response_deserializer=lightning__pb2.TransactionDetails.FromString,
                )
        self.EstimateFee = channel.unary_unary(
                '/lnrpc.Lightning/EstimateFee',
                request_serializer=lightning__pb2.EstimateFeeRequest.SerializeToString,
                response_deserializer=lightning__pb2.EstimateFeeResponse.FromString,
                )
        self.SendCoins = channel.unary_unary(
                '/lnrpc.Lightning/SendCoins',
                request_serializer=lightning__pb2.SendCoinsRequest.SerializeToString,
                response_deserializer=lightning__pb2.SendCoinsResponse.FromString,
                )
        self.ListUnspent = channel.unary_unary(
                '/lnrpc.Lightning/ListUnspent',
                request_serializer=lightning__pb2.ListUnspentRequest.SerializeToString,
                response_deserializer=lightning__pb2.ListUnspentResponse.FromString,
                )
        self.SubscribeTransactions = channel.unary_stream(
                '/lnrpc.Lightning/SubscribeTransactions',
                request_serializer=lightning__pb2.GetTransactionsRequest.SerializeToString,
                response_deserializer=lightning__pb2.Transaction.FromString,
                )
        self.SendMany = channel.unary_unary(
                '/lnrpc.Lightning/SendMany',
                request_serializer=lightning__pb2.SendManyRequest.SerializeToString,
                response_deserializer=lightning__pb2.SendManyResponse.FromString,
                )
        self.NewAddress = channel.unary_unary(
                '/lnrpc.Lightning/NewAddress',
                request_serializer=lightning__pb2.NewAddressRequest.SerializeToString,
                response_deserializer=lightning__pb2.NewAddressResponse.FromString,
                )
        self.SignMessage = channel.unary_unary(
                '/lnrpc.Lightning/SignMessage',
                request_serializer=lightning__pb2.SignMessageRequest.SerializeToString,
                response_deserializer=lightning__pb2.SignMessageResponse.FromString,
                )
        self.VerifyMessage = channel.unary_unary(
                '/lnrpc.Lightning/VerifyMessage',
                request_serializer=lightning__pb2.VerifyMessageRequest.SerializeToString,
                response_deserializer=lightning__pb2.VerifyMessageResponse.FromString,
                )
        self.ConnectPeer = channel.unary_unary(
                '/lnrpc.Lightning/ConnectPeer',
                request_serializer=lightning__pb2.ConnectPeerRequest.SerializeToString,
                response_deserializer=lightning__pb2.ConnectPeerResponse.FromString,
                )
        self.DisconnectPeer = channel.unary_unary(
                '/lnrpc.Lightning/DisconnectPeer',
                request_serializer=lightning__pb2.DisconnectPeerRequest.SerializeToString,
                response_deserializer=lightning__pb2.DisconnectPeerResponse.FromString,
                )
        self.ListPeers = channel.unary_unary(
                '/lnrpc.Lightning/ListPeers',
                request_serializer=lightning__pb2.ListPeersRequest.SerializeToString,
                response_deserializer=lightning__pb2.ListPeersResponse.FromString,
                )
        self.SubscribePeerEvents = channel.unary_stream(
                '/lnrpc.Lightning/SubscribePeerEvents',
                request_serializer=lightning__pb2.PeerEventSubscription.SerializeToString,
                response_deserializer=lightning__pb2.PeerEvent.FromString,
                )
        self.GetInfo = channel.unary_unary(
                '/lnrpc.Lightning/GetInfo',
                request_serializer=lightning__pb2.GetInfoRequest.SerializeToString,
                response_deserializer=lightning__pb2.GetInfoResponse.FromString,
                )
        self.GetRecoveryInfo = channel.unary_unary(
                '/lnrpc.Lightning/GetRecoveryInfo',
                request_serializer=lightning__pb2.GetRecoveryInfoRequest.SerializeToString,
                response_deserializer=lightning__pb2.GetRecoveryInfoResponse.FromString,
                )
        self.PendingChannels = channel.unary_unary(
                '/lnrpc.Lightning/PendingChannels',
                request_serializer=lightning__pb2.PendingChannelsRequest.SerializeToString,
                response_deserializer=lightning__pb2.PendingChannelsResponse.FromString,
                )
        self.ListChannels = channel.unary_unary(
                '/lnrpc.Lightning/ListChannels',
                request_serializer=lightning__pb2.ListChannelsRequest.SerializeToString,
                response_deserializer=lightning__pb2.ListChannelsResponse.FromString,
                )
        self.SubscribeChannelEvents = channel.unary_stream(
                '/lnrpc.Lightning/SubscribeChannelEvents',
                request_serializer=lightning__pb2.ChannelEventSubscription.SerializeToString,
                response_deserializer=lightning__pb2.ChannelEventUpdate.FromString,
                )
        self.ClosedChannels = channel.unary_unary(
                '/lnrpc.Lightning/ClosedChannels',
                request_serializer=lightning__pb2.ClosedChannelsRequest.SerializeToString,
                response_deserializer=lightning__pb2.ClosedChannelsResponse.FromString,
                )
        self.OpenChannelSync = channel.unary_unary(
                '/lnrpc.Lightning/OpenChannelSync',
                request_serializer=lightning__pb2.OpenChannelRequest.SerializeToString,
                response_deserializer=lightning__pb2.ChannelPoint.FromString,
                )
        self.OpenChannel = channel.unary_stream(
                '/lnrpc.Lightning/OpenChannel',
                request_serializer=lightning__pb2.OpenChannelRequest.SerializeToString,
                response_deserializer=lightning__pb2.OpenStatusUpdate.FromString,
                )
        self.BatchOpenChannel = channel.unary_unary(
                '/lnrpc.Lightning/BatchOpenChannel',
                request_serializer=lightning__pb2.BatchOpenChannelRequest.SerializeToString,
                response_deserializer=lightning__pb2.BatchOpenChannelResponse.FromString,
                )
        self.FundingStateStep = channel.unary_unary(
                '/lnrpc.Lightning/FundingStateStep',
                request_serializer=lightning__pb2.FundingTransitionMsg.SerializeToString,
                response_deserializer=lightning__pb2.FundingStateStepResp.FromString,
                )
        self.ChannelAcceptor = channel.stream_stream(
                '/lnrpc.Lightning/ChannelAcceptor',
                request_serializer=lightning__pb2.ChannelAcceptResponse.SerializeToString,
                response_deserializer=lightning__pb2.ChannelAcceptRequest.FromString,
                )
        self.CloseChannel = channel.unary_stream(
                '/lnrpc.Lightning/CloseChannel',
                request_serializer=lightning__pb2.CloseChannelRequest.SerializeToString,
                response_deserializer=lightning__pb2.CloseStatusUpdate.FromString,
                )
        self.AbandonChannel = channel.unary_unary(
                '/lnrpc.Lightning/AbandonChannel',
                request_serializer=lightning__pb2.AbandonChannelRequest.SerializeToString,
                response_deserializer=lightning__pb2.AbandonChannelResponse.FromString,
                )
        self.SendPayment = channel.stream_stream(
                '/lnrpc.Lightning/SendPayment',
                request_serializer=lightning__pb2.SendRequest.SerializeToString,
                response_deserializer=lightning__pb2.SendResponse.FromString,
                )
        self.SendPaymentSync = channel.unary_unary(
                '/lnrpc.Lightning/SendPaymentSync',
                request_serializer=lightning__pb2.SendRequest.SerializeToString,
                response_deserializer=lightning__pb2.SendResponse.FromString,
                )
        self.SendToRoute = channel.stream_stream(
                '/lnrpc.Lightning/SendToRoute',
                request_serializer=lightning__pb2.SendToRouteRequest.SerializeToString,
                response_deserializer=lightning__pb2.SendResponse.FromString,
                )
        self.SendToRouteSync = channel.unary_unary(
                '/lnrpc.Lightning/SendToRouteSync',
                request_serializer=lightning__pb2.SendToRouteRequest.SerializeToString,
                response_deserializer=lightning__pb2.SendResponse.FromString,
                )
        self.AddInvoice = channel.unary_unary(
                '/lnrpc.Lightning/AddInvoice',
                request_serializer=lightning__pb2.Invoice.SerializeToString,
                response_deserializer=lightning__pb2.AddInvoiceResponse.FromString,
                )
        self.ListInvoices = channel.unary_unary(
                '/lnrpc.Lightning/ListInvoices',
                request_serializer=lightning__pb2.ListInvoiceRequest.SerializeToString,
                response_deserializer=lightning__pb2.ListInvoiceResponse.FromString,
                )
        self.LookupInvoice = channel.unary_unary(
                '/lnrpc.Lightning/LookupInvoice',
                request_serializer=lightning__pb2.PaymentHash.SerializeToString,
                response_deserializer=lightning__pb2.Invoice.FromString,
                )
        self.SubscribeInvoices = channel.unary_stream(
                '/lnrpc.Lightning/SubscribeInvoices',
                request_serializer=lightning__pb2.InvoiceSubscription.SerializeToString,
                response_deserializer=lightning__pb2.Invoice.FromString,
                )
        self.DecodePayReq = channel.unary_unary(
                '/lnrpc.Lightning/DecodePayReq',
                request_serializer=lightning__pb2.PayReqString.SerializeToString,
                response_deserializer=lightning__pb2.PayReq.FromString,
                )
        self.ListPayments = channel.unary_unary(
                '/lnrpc.Lightning/ListPayments',
                request_serializer=lightning__pb2.ListPaymentsRequest.SerializeToString,
                response_deserializer=lightning__pb2.ListPaymentsResponse.FromString,
                )
        self.DeletePayment = channel.unary_unary(
                '/lnrpc.Lightning/DeletePayment',
                request_serializer=lightning__pb2.DeletePaymentRequest.SerializeToString,
                response_deserializer=lightning__pb2.DeletePaymentResponse.FromString,
                )
        self.DeleteAllPayments = channel.unary_unary(
                '/lnrpc.Lightning/DeleteAllPayments',
                request_serializer=lightning__pb2.DeleteAllPaymentsRequest.SerializeToString,
                response_deserializer=lightning__pb2.DeleteAllPaymentsResponse.FromString,
                )
        self.DescribeGraph = channel.unary_unary(
                '/lnrpc.Lightning/DescribeGraph',
                request_serializer=lightning__pb2.ChannelGraphRequest.SerializeToString,
                response_deserializer=lightning__pb2.ChannelGraph.FromString,
                )
        self.GetNodeMetrics = channel.unary_unary(
                '/lnrpc.Lightning/GetNodeMetrics',
                request_serializer=lightning__pb2.NodeMetricsRequest.SerializeToString,
                response_deserializer=lightning__pb2.NodeMetricsResponse.FromString,
                )
        self.GetChanInfo = channel.unary_unary(
                '/lnrpc.Lightning/GetChanInfo',
                request_serializer=lightning__pb2.ChanInfoRequest.SerializeToString,
                response_deserializer=lightning__pb2.ChannelEdge.FromString,
                )
        self.GetNodeInfo = channel.unary_unary(
                '/lnrpc.Lightning/GetNodeInfo',
                request_serializer=lightning__pb2.NodeInfoRequest.SerializeToString,
                response_deserializer=lightning__pb2.NodeInfo.FromString,
                )
        self.QueryRoutes = channel.unary_unary(
                '/lnrpc.Lightning/QueryRoutes',
                request_serializer=lightning__pb2.QueryRoutesRequest.SerializeToString,
                response_deserializer=lightning__pb2.QueryRoutesResponse.FromString,
                )
        self.GetNetworkInfo = channel.unary_unary(
                '/lnrpc.Lightning/GetNetworkInfo',
                request_serializer=lightning__pb2.NetworkInfoRequest.SerializeToString,
                response_deserializer=lightning__pb2.NetworkInfo.FromString,
                )
        self.StopDaemon = channel.unary_unary(
                '/lnrpc.Lightning/StopDaemon',
                request_serializer=lightning__pb2.StopRequest.SerializeToString,
                response_deserializer=lightning__pb2.StopResponse.FromString,
                )
        self.SubscribeChannelGraph = channel.unary_stream(
                '/lnrpc.Lightning/SubscribeChannelGraph',
                request_serializer=lightning__pb2.GraphTopologySubscription.SerializeToString,
                response_deserializer=lightning__pb2.GraphTopologyUpdate.FromString,
                )
        self.DebugLevel = channel.unary_unary(
                '/lnrpc.Lightning/DebugLevel',
                request_serializer=lightning__pb2.DebugLevelRequest.SerializeToString,
                response_deserializer=lightning__pb2.DebugLevelResponse.FromString,
                )
        self.FeeReport = channel.unary_unary(
                '/lnrpc.Lightning/FeeReport',
                request_serializer=lightning__pb2.FeeReportRequest.SerializeToString,
                response_deserializer=lightning__pb2.FeeReportResponse.FromString,
                )
        self.UpdateChannelPolicy = channel.unary_unary(
                '/lnrpc.Lightning/UpdateChannelPolicy',
                request_serializer=lightning__pb2.PolicyUpdateRequest.SerializeToString,
                response_deserializer=lightning__pb2.PolicyUpdateResponse.FromString,
                )
        self.ForwardingHistory = channel.unary_unary(
                '/lnrpc.Lightning/ForwardingHistory',
                request_serializer=lightning__pb2.ForwardingHistoryRequest.SerializeToString,
                response_deserializer=lightning__pb2.ForwardingHistoryResponse.FromString,
                )
        self.ExportChannelBackup = channel.unary_unary(
                '/lnrpc.Lightning/ExportChannelBackup',
                request_serializer=lightning__pb2.ExportChannelBackupRequest.SerializeToString,
                response_deserializer=lightning__pb2.ChannelBackup.FromString,
                )
        self.ExportAllChannelBackups = channel.unary_unary(
                '/lnrpc.Lightning/ExportAllChannelBackups',
                request_serializer=lightning__pb2.ChanBackupExportRequest.SerializeToString,
                response_deserializer=lightning__pb2.ChanBackupSnapshot.FromString,
                )
        self.VerifyChanBackup = channel.unary_unary(
                '/lnrpc.Lightning/VerifyChanBackup',
                request_serializer=lightning__pb2.ChanBackupSnapshot.SerializeToString,
                response_deserializer=lightning__pb2.VerifyChanBackupResponse.FromString,
                )
        self.RestoreChannelBackups = channel.unary_unary(
                '/lnrpc.Lightning/RestoreChannelBackups',
                request_serializer=lightning__pb2.RestoreChanBackupRequest.SerializeToString,
                response_deserializer=lightning__pb2.RestoreBackupResponse.FromString,
                )
        self.SubscribeChannelBackups = channel.unary_stream(
                '/lnrpc.Lightning/SubscribeChannelBackups',
                request_serializer=lightning__pb2.ChannelBackupSubscription.SerializeToString,
                response_deserializer=lightning__pb2.ChanBackupSnapshot.FromString,
                )
        self.BakeMacaroon = channel.unary_unary(
                '/lnrpc.Lightning/BakeMacaroon',
                request_serializer=lightning__pb2.BakeMacaroonRequest.SerializeToString,
                response_deserializer=lightning__pb2.BakeMacaroonResponse.FromString,
                )
        self.ListMacaroonIDs = channel.unary_unary(
                '/lnrpc.Lightning/ListMacaroonIDs',
                request_serializer=lightning__pb2.ListMacaroonIDsRequest.SerializeToString,
                response_deserializer=lightning__pb2.ListMacaroonIDsResponse.FromString,
                )
        self.DeleteMacaroonID = channel.unary_unary(
                '/lnrpc.Lightning/DeleteMacaroonID',
                request_serializer=lightning__pb2.DeleteMacaroonIDRequest.SerializeToString,
                response_deserializer=lightning__pb2.DeleteMacaroonIDResponse.FromString,
                )
        self.ListPermissions = channel.unary_unary(
                '/lnrpc.Lightning/ListPermissions',
                request_serializer=lightning__pb2.ListPermissionsRequest.SerializeToString,
                response_deserializer=lightning__pb2.ListPermissionsResponse.FromString,
                )
        self.CheckMacaroonPermissions = channel.unary_unary(
                '/lnrpc.Lightning/CheckMacaroonPermissions',
                request_serializer=lightning__pb2.CheckMacPermRequest.SerializeToString,
                response_deserializer=lightning__pb2.CheckMacPermResponse.FromString,
                )
        self.RegisterRPCMiddleware = channel.stream_stream(
                '/lnrpc.Lightning/RegisterRPCMiddleware',
                request_serializer=lightning__pb2.RPCMiddlewareResponse.SerializeToString,
                response_deserializer=lightning__pb2.RPCMiddlewareRequest.FromString,
                )
        self.SendCustomMessage = channel.unary_unary(
                '/lnrpc.Lightning/SendCustomMessage',
                request_serializer=lightning__pb2.SendCustomMessageRequest.SerializeToString,
                response_deserializer=lightning__pb2.SendCustomMessageResponse.FromString,
                )
        self.SubscribeCustomMessages = channel.unary_stream(
                '/lnrpc.Lightning/SubscribeCustomMessages',
                request_serializer=lightning__pb2.SubscribeCustomMessagesRequest.SerializeToString,
                response_deserializer=lightning__pb2.CustomMessage.FromString,
                )


class LightningServicer(object):
    """
    Comments in this file will be directly parsed into the API
    Documentation as descriptions of the associated method, message, or field.
    These descriptions should go right above the definition of the object, and
    can be in either block or // comment format.

    An RPC method can be matched to an lncli command by placing a line in the
    beginning of the description in exactly the following format:
    lncli: `methodname`

    Failure to specify the exact name of the command will cause documentation
    generation to fail.

    More information on how exactly the gRPC documentation is generated from
    this proto file can be found here:
    https://github.com/lightninglabs/lightning-api

    Lightning is the main RPC server of the daemon.
    """

    def WalletBalance(self, request, context):
        """lncli: `walletbalance`
        WalletBalance returns total unspent outputs(confirmed and unconfirmed), all
        confirmed unspent outputs and all unconfirmed unspent outputs under control
        of the wallet.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ChannelBalance(self, request, context):
        """lncli: `channelbalance`
        ChannelBalance returns a report on the total funds across all open channels,
        categorized in local/remote, pending local/remote and unsettled local/remote
        balances.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTransactions(self, request, context):
        """lncli: `listchaintxns`
        GetTransactions returns a list describing all the known transactions
        relevant to the wallet.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def EstimateFee(self, request, context):
        """lncli: `estimatefee`
        EstimateFee asks the chain backend to estimate the fee rate and total fees
        for a transaction that pays to multiple specified outputs.

        When using REST, the `AddrToAmount` map type can be set by appending
        `&AddrToAmount[<address>]=<amount_to_send>` to the URL. Unfortunately this
        map type doesn't appear in the REST API documentation because of a bug in
        the grpc-gateway library.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendCoins(self, request, context):
        """lncli: `sendcoins`
        SendCoins executes a request to send coins to a particular address. Unlike
        SendMany, this RPC call only allows creating a single output at a time. If
        neither target_conf, or sat_per_vbyte are set, then the internal wallet will
        consult its fee model to determine a fee for the default confirmation
        target.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListUnspent(self, request, context):
        """lncli: `listunspent`
        Deprecated, use walletrpc.ListUnspent instead.

        ListUnspent returns a list of all utxos spendable by the wallet with a
        number of confirmations between the specified minimum and maximum.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SubscribeTransactions(self, request, context):
        """
        SubscribeTransactions creates a uni-directional stream from the server to
        the client in which any newly discovered transactions relevant to the
        wallet are sent over.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendMany(self, request, context):
        """lncli: `sendmany`
        SendMany handles a request for a transaction that creates multiple specified
        outputs in parallel. If neither target_conf, or sat_per_vbyte are set, then
        the internal wallet will consult its fee model to determine a fee for the
        default confirmation target.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def NewAddress(self, request, context):
        """lncli: `newaddress`
        NewAddress creates a new address under control of the local wallet.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SignMessage(self, request, context):
        """lncli: `signmessage`
        SignMessage signs a message with this node's private key. The returned
        signature string is `zbase32` encoded and pubkey recoverable, meaning that
        only the message digest and signature are needed for verification.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VerifyMessage(self, request, context):
        """lncli: `verifymessage`
        VerifyMessage verifies a signature over a msg. The signature must be
        zbase32 encoded and signed by an active node in the resident node's
        channel database. In addition to returning the validity of the signature,
        VerifyMessage also returns the recovered pubkey from the signature.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ConnectPeer(self, request, context):
        """lncli: `connect`
        ConnectPeer attempts to establish a connection to a remote peer. This is at
        the networking level, and is used for communication between nodes. This is
        distinct from establishing a channel with a peer.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DisconnectPeer(self, request, context):
        """lncli: `disconnect`
        DisconnectPeer attempts to disconnect one peer from another identified by a
        given pubKey. In the case that we currently have a pending or active channel
        with the target peer, then this action will be not be allowed.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListPeers(self, request, context):
        """lncli: `listpeers`
        ListPeers returns a verbose listing of all currently active peers.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SubscribePeerEvents(self, request, context):
        """
        SubscribePeerEvents creates a uni-directional stream from the server to
        the client in which any events relevant to the state of peers are sent
        over. Events include peers going online and offline.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetInfo(self, request, context):
        """lncli: `getinfo`
        GetInfo returns general information concerning the lightning node including
        it's identity pubkey, alias, the chains it is connected to, and information
        concerning the number of open+pending channels.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRecoveryInfo(self, request, context):
        """* lncli: `getrecoveryinfo`
        GetRecoveryInfo returns information concerning the recovery mode including
        whether it's in a recovery mode, whether the recovery is finished, and the
        progress made so far.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PendingChannels(self, request, context):
        """TODO(roasbeef): merge with below with bool?

        lncli: `pendingchannels`
        PendingChannels returns a list of all the channels that are currently
        considered "pending". A channel is pending if it has finished the funding
        workflow and is waiting for confirmations for the funding txn, or is in the
        process of closure, either initiated cooperatively or non-cooperatively.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListChannels(self, request, context):
        """lncli: `listchannels`
        ListChannels returns a description of all the open channels that this node
        is a participant in.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SubscribeChannelEvents(self, request, context):
        """
        SubscribeChannelEvents creates a uni-directional stream from the server to
        the client in which any updates relevant to the state of the channels are
        sent over. Events include new active channels, inactive channels, and closed
        channels.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClosedChannels(self, request, context):
        """lncli: `closedchannels`
        ClosedChannels returns a description of all the closed channels that
        this node was a participant in.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def OpenChannelSync(self, request, context):
        """
        OpenChannelSync is a synchronous version of the OpenChannel RPC call. This
        call is meant to be consumed by clients to the REST proxy. As with all
        other sync calls, all byte slices are intended to be populated as hex
        encoded strings.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def OpenChannel(self, request, context):
        """lncli: `openchannel`
        OpenChannel attempts to open a singly funded channel specified in the
        request to a remote peer. Users are able to specify a target number of
        blocks that the funding transaction should be confirmed in, or a manual fee
        rate to us for the funding transaction. If neither are specified, then a
        lax block confirmation target is used. Each OpenStatusUpdate will return
        the pending channel ID of the in-progress channel. Depending on the
        arguments specified in the OpenChannelRequest, this pending channel ID can
        then be used to manually progress the channel funding flow.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BatchOpenChannel(self, request, context):
        """lncli: `batchopenchannel`
        BatchOpenChannel attempts to open multiple single-funded channels in a
        single transaction in an atomic way. This means either all channel open
        requests succeed at once or all attempts are aborted if any of them fail.
        This is the safer variant of using PSBTs to manually fund a batch of
        channels through the OpenChannel RPC.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FundingStateStep(self, request, context):
        """
        FundingStateStep is an advanced funding related call that allows the caller
        to either execute some preparatory steps for a funding workflow, or
        manually progress a funding workflow. The primary way a funding flow is
        identified is via its pending channel ID. As an example, this method can be
        used to specify that we're expecting a funding flow for a particular
        pending channel ID, for which we need to use specific parameters.
        Alternatively, this can be used to interactively drive PSBT signing for
        funding for partially complete funding transactions.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ChannelAcceptor(self, request_iterator, context):
        """
        ChannelAcceptor dispatches a bi-directional streaming RPC in which
        OpenChannel requests are sent to the client and the client responds with
        a boolean that tells LND whether or not to accept the channel. This allows
        node operators to specify their own criteria for accepting inbound channels
        through a single persistent connection.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CloseChannel(self, request, context):
        """lncli: `closechannel`
        CloseChannel attempts to close an active channel identified by its channel
        outpoint (ChannelPoint). The actions of this method can additionally be
        augmented to attempt a force close after a timeout period in the case of an
        inactive peer. If a non-force close (cooperative closure) is requested,
        then the user can specify either a target number of blocks until the
        closure transaction is confirmed, or a manual fee rate. If neither are
        specified, then a default lax, block confirmation target is used.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AbandonChannel(self, request, context):
        """lncli: `abandonchannel`
        AbandonChannel removes all channel state from the database except for a
        close summary. This method can be used to get rid of permanently unusable
        channels due to bugs fixed in newer versions of lnd. This method can also be
        used to remove externally funded channels where the funding transaction was
        never broadcast. Only available for non-externally funded channels in dev
        build.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendPayment(self, request_iterator, context):
        """lncli: `sendpayment`
        Deprecated, use routerrpc.SendPaymentV2. SendPayment dispatches a
        bi-directional streaming RPC for sending payments through the Lightning
        Network. A single RPC invocation creates a persistent bi-directional
        stream allowing clients to rapidly send payments through the Lightning
        Network with a single persistent connection.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendPaymentSync(self, request, context):
        """
        SendPaymentSync is the synchronous non-streaming version of SendPayment.
        This RPC is intended to be consumed by clients of the REST proxy.
        Additionally, this RPC expects the destination's public key and the payment
        hash (if any) to be encoded as hex strings.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendToRoute(self, request_iterator, context):
        """lncli: `sendtoroute`
        Deprecated, use routerrpc.SendToRouteV2. SendToRoute is a bi-directional
        streaming RPC for sending payment through the Lightning Network. This
        method differs from SendPayment in that it allows users to specify a full
        route manually. This can be used for things like rebalancing, and atomic
        swaps.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendToRouteSync(self, request, context):
        """
        SendToRouteSync is a synchronous version of SendToRoute. It Will block
        until the payment either fails or succeeds.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddInvoice(self, request, context):
        """lncli: `addinvoice`
        AddInvoice attempts to add a new invoice to the invoice database. Any
        duplicated invoices are rejected, therefore all invoices *must* have a
        unique payment preimage.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListInvoices(self, request, context):
        """lncli: `listinvoices`
        ListInvoices returns a list of all the invoices currently stored within the
        database. Any active debug invoices are ignored. It has full support for
        paginated responses, allowing users to query for specific invoices through
        their add_index. This can be done by using either the first_index_offset or
        last_index_offset fields included in the response as the index_offset of the
        next request. By default, the first 100 invoices created will be returned.
        Backwards pagination is also supported through the Reversed flag.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def LookupInvoice(self, request, context):
        """lncli: `lookupinvoice`
        LookupInvoice attempts to look up an invoice according to its payment hash.
        The passed payment hash *must* be exactly 32 bytes, if not, an error is
        returned.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SubscribeInvoices(self, request, context):
        """
        SubscribeInvoices returns a uni-directional stream (server -> client) for
        notifying the client of newly added/settled invoices. The caller can
        optionally specify the add_index and/or the settle_index. If the add_index
        is specified, then we'll first start by sending add invoice events for all
        invoices with an add_index greater than the specified value. If the
        settle_index is specified, the next, we'll send out all settle events for
        invoices with a settle_index greater than the specified value. One or both
        of these fields can be set. If no fields are set, then we'll only send out
        the latest add/settle events.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DecodePayReq(self, request, context):
        """lncli: `decodepayreq`
        DecodePayReq takes an encoded payment request string and attempts to decode
        it, returning a full description of the conditions encoded within the
        payment request.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListPayments(self, request, context):
        """lncli: `listpayments`
        ListPayments returns a list of all outgoing payments.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeletePayment(self, request, context):
        """
        DeletePayment deletes an outgoing payment from DB. Note that it will not
        attempt to delete an In-Flight payment, since that would be unsafe.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteAllPayments(self, request, context):
        """
        DeleteAllPayments deletes all outgoing payments from DB. Note that it will
        not attempt to delete In-Flight payments, since that would be unsafe.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DescribeGraph(self, request, context):
        """lncli: `describegraph`
        DescribeGraph returns a description of the latest graph state from the
        point of view of the node. The graph information is partitioned into two
        components: all the nodes/vertexes, and all the edges that connect the
        vertexes themselves. As this is a directed graph, the edges also contain
        the node directional specific routing policy which includes: the time lock
        delta, fee information, etc.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetNodeMetrics(self, request, context):
        """lncli: `getnodemetrics`
        GetNodeMetrics returns node metrics calculated from the graph. Currently
        the only supported metric is betweenness centrality of individual nodes.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetChanInfo(self, request, context):
        """lncli: `getchaninfo`
        GetChanInfo returns the latest authenticated network announcement for the
        given channel identified by its channel ID: an 8-byte integer which
        uniquely identifies the location of transaction's funding output within the
        blockchain.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetNodeInfo(self, request, context):
        """lncli: `getnodeinfo`
        GetNodeInfo returns the latest advertised, aggregated, and authenticated
        channel information for the specified node identified by its public key.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryRoutes(self, request, context):
        """lncli: `queryroutes`
        QueryRoutes attempts to query the daemon's Channel Router for a possible
        route to a target destination capable of carrying a specific amount of
        satoshis. The returned route contains the full details required to craft and
        send an HTLC, also including the necessary information that should be
        present within the Sphinx packet encapsulated within the HTLC.

        When using REST, the `dest_custom_records` map type can be set by appending
        `&dest_custom_records[<record_number>]=<record_data_base64_url_encoded>`
        to the URL. Unfortunately this map type doesn't appear in the REST API
        documentation because of a bug in the grpc-gateway library.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetNetworkInfo(self, request, context):
        """lncli: `getnetworkinfo`
        GetNetworkInfo returns some basic stats about the known channel graph from
        the point of view of the node.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StopDaemon(self, request, context):
        """lncli: `stop`
        StopDaemon will send a shutdown request to the interrupt handler, triggering
        a graceful shutdown of the daemon.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SubscribeChannelGraph(self, request, context):
        """
        SubscribeChannelGraph launches a streaming RPC that allows the caller to
        receive notifications upon any changes to the channel graph topology from
        the point of view of the responding node. Events notified include: new
        nodes coming online, nodes updating their authenticated attributes, new
        channels being advertised, updates in the routing policy for a directional
        channel edge, and when channels are closed on-chain.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DebugLevel(self, request, context):
        """lncli: `debuglevel`
        DebugLevel allows a caller to programmatically set the logging verbosity of
        lnd. The logging can be targeted according to a coarse daemon-wide logging
        level, or in a granular fashion to specify the logging for a target
        sub-system.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FeeReport(self, request, context):
        """lncli: `feereport`
        FeeReport allows the caller to obtain a report detailing the current fee
        schedule enforced by the node globally for each channel.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateChannelPolicy(self, request, context):
        """lncli: `updatechanpolicy`
        UpdateChannelPolicy allows the caller to update the fee schedule and
        channel policies for all channels globally, or a particular channel.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ForwardingHistory(self, request, context):
        """lncli: `fwdinghistory`
        ForwardingHistory allows the caller to query the htlcswitch for a record of
        all HTLCs forwarded within the target time range, and integer offset
        within that time range, for a maximum number of events. If no maximum number
        of events is specified, up to 100 events will be returned. If no time-range
        is specified, then events will be returned in the order that they occured.

        A list of forwarding events are returned. The size of each forwarding event
        is 40 bytes, and the max message size able to be returned in gRPC is 4 MiB.
        As a result each message can only contain 50k entries. Each response has
        the index offset of the last entry. The index offset can be provided to the
        request to allow the caller to skip a series of records.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ExportChannelBackup(self, request, context):
        """lncli: `exportchanbackup`
        ExportChannelBackup attempts to return an encrypted static channel backup
        for the target channel identified by it channel point. The backup is
        encrypted with a key generated from the aezeed seed of the user. The
        returned backup can either be restored using the RestoreChannelBackup
        method once lnd is running, or via the InitWallet and UnlockWallet methods
        from the WalletUnlocker service.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ExportAllChannelBackups(self, request, context):
        """
        ExportAllChannelBackups returns static channel backups for all existing
        channels known to lnd. A set of regular singular static channel backups for
        each channel are returned. Additionally, a multi-channel backup is returned
        as well, which contains a single encrypted blob containing the backups of
        each channel.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VerifyChanBackup(self, request, context):
        """
        VerifyChanBackup allows a caller to verify the integrity of a channel backup
        snapshot. This method will accept either a packed Single or a packed Multi.
        Specifying both will result in an error.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RestoreChannelBackups(self, request, context):
        """lncli: `restorechanbackup`
        RestoreChannelBackups accepts a set of singular channel backups, or a
        single encrypted multi-chan backup and attempts to recover any funds
        remaining within the channel. If we are able to unpack the backup, then the
        new channel will be shown under listchannels, as well as pending channels.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SubscribeChannelBackups(self, request, context):
        """
        SubscribeChannelBackups allows a client to sub-subscribe to the most up to
        date information concerning the state of all channel backups. Each time a
        new channel is added, we return the new set of channels, along with a
        multi-chan backup containing the backup info for all channels. Each time a
        channel is closed, we send a new update, which contains new new chan back
        ups, but the updated set of encrypted multi-chan backups with the closed
        channel(s) removed.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BakeMacaroon(self, request, context):
        """lncli: `bakemacaroon`
        BakeMacaroon allows the creation of a new macaroon with custom read and
        write permissions. No first-party caveats are added since this can be done
        offline.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListMacaroonIDs(self, request, context):
        """lncli: `listmacaroonids`
        ListMacaroonIDs returns all root key IDs that are in use.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteMacaroonID(self, request, context):
        """lncli: `deletemacaroonid`
        DeleteMacaroonID deletes the specified macaroon ID and invalidates all
        macaroons derived from that ID.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListPermissions(self, request, context):
        """lncli: `listpermissions`
        ListPermissions lists all RPC method URIs and their required macaroon
        permissions to access them.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CheckMacaroonPermissions(self, request, context):
        """
        CheckMacaroonPermissions checks whether a request follows the constraints
        imposed on the macaroon and that the macaroon is authorized to follow the
        provided permissions.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RegisterRPCMiddleware(self, request_iterator, context):
        """
        RegisterRPCMiddleware adds a new gRPC middleware to the interceptor chain. A
        gRPC middleware is software component external to lnd that aims to add
        additional business logic to lnd by observing/intercepting/validating
        incoming gRPC client requests and (if needed) replacing/overwriting outgoing
        messages before they're sent to the client. When registering the middleware
        must identify itself and indicate what custom macaroon caveats it wants to
        be responsible for. Only requests that contain a macaroon with that specific
        custom caveat are then sent to the middleware for inspection. The other
        option is to register for the read-only mode in which all requests/responses
        are forwarded for interception to the middleware but the middleware is not
        allowed to modify any responses. As a security measure, _no_ middleware can
        modify responses for requests made with _unencumbered_ macaroons!
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendCustomMessage(self, request, context):
        """lncli: `sendcustom`
        SendCustomMessage sends a custom peer message.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SubscribeCustomMessages(self, request, context):
        """lncli: `subscribecustom`
        SubscribeCustomMessages subscribes to a stream of incoming custom peer
        messages.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_LightningServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'WalletBalance': grpc.unary_unary_rpc_method_handler(
                    servicer.WalletBalance,
                    request_deserializer=lightning__pb2.WalletBalanceRequest.FromString,
                    response_serializer=lightning__pb2.WalletBalanceResponse.SerializeToString,
            ),
            'ChannelBalance': grpc.unary_unary_rpc_method_handler(
                    servicer.ChannelBalance,
                    request_deserializer=lightning__pb2.ChannelBalanceRequest.FromString,
                    response_serializer=lightning__pb2.ChannelBalanceResponse.SerializeToString,
            ),
            'GetTransactions': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTransactions,
                    request_deserializer=lightning__pb2.GetTransactionsRequest.FromString,
                    response_serializer=lightning__pb2.TransactionDetails.SerializeToString,
            ),
            'EstimateFee': grpc.unary_unary_rpc_method_handler(
                    servicer.EstimateFee,
                    request_deserializer=lightning__pb2.EstimateFeeRequest.FromString,
                    response_serializer=lightning__pb2.EstimateFeeResponse.SerializeToString,
            ),
            'SendCoins': grpc.unary_unary_rpc_method_handler(
                    servicer.SendCoins,
                    request_deserializer=lightning__pb2.SendCoinsRequest.FromString,
                    response_serializer=lightning__pb2.SendCoinsResponse.SerializeToString,
            ),
            'ListUnspent': grpc.unary_unary_rpc_method_handler(
                    servicer.ListUnspent,
                    request_deserializer=lightning__pb2.ListUnspentRequest.FromString,
                    response_serializer=lightning__pb2.ListUnspentResponse.SerializeToString,
            ),
            'SubscribeTransactions': grpc.unary_stream_rpc_method_handler(
                    servicer.SubscribeTransactions,
                    request_deserializer=lightning__pb2.GetTransactionsRequest.FromString,
                    response_serializer=lightning__pb2.Transaction.SerializeToString,
            ),
            'SendMany': grpc.unary_unary_rpc_method_handler(
                    servicer.SendMany,
                    request_deserializer=lightning__pb2.SendManyRequest.FromString,
                    response_serializer=lightning__pb2.SendManyResponse.SerializeToString,
            ),
            'NewAddress': grpc.unary_unary_rpc_method_handler(
                    servicer.NewAddress,
                    request_deserializer=lightning__pb2.NewAddressRequest.FromString,
                    response_serializer=lightning__pb2.NewAddressResponse.SerializeToString,
            ),
            'SignMessage': grpc.unary_unary_rpc_method_handler(
                    servicer.SignMessage,
                    request_deserializer=lightning__pb2.SignMessageRequest.FromString,
                    response_serializer=lightning__pb2.SignMessageResponse.SerializeToString,
            ),
            'VerifyMessage': grpc.unary_unary_rpc_method_handler(
                    servicer.VerifyMessage,
                    request_deserializer=lightning__pb2.VerifyMessageRequest.FromString,
                    response_serializer=lightning__pb2.VerifyMessageResponse.SerializeToString,
            ),
            'ConnectPeer': grpc.unary_unary_rpc_method_handler(
                    servicer.ConnectPeer,
                    request_deserializer=lightning__pb2.ConnectPeerRequest.FromString,
                    response_serializer=lightning__pb2.ConnectPeerResponse.SerializeToString,
            ),
            'DisconnectPeer': grpc.unary_unary_rpc_method_handler(
                    servicer.DisconnectPeer,
                    request_deserializer=lightning__pb2.DisconnectPeerRequest.FromString,
                    response_serializer=lightning__pb2.DisconnectPeerResponse.SerializeToString,
            ),
            'ListPeers': grpc.unary_unary_rpc_method_handler(
                    servicer.ListPeers,
                    request_deserializer=lightning__pb2.ListPeersRequest.FromString,
                    response_serializer=lightning__pb2.ListPeersResponse.SerializeToString,
            ),
            'SubscribePeerEvents': grpc.unary_stream_rpc_method_handler(
                    servicer.SubscribePeerEvents,
                    request_deserializer=lightning__pb2.PeerEventSubscription.FromString,
                    response_serializer=lightning__pb2.PeerEvent.SerializeToString,
            ),
            'GetInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetInfo,
                    request_deserializer=lightning__pb2.GetInfoRequest.FromString,
                    response_serializer=lightning__pb2.GetInfoResponse.SerializeToString,
            ),
            'GetRecoveryInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRecoveryInfo,
                    request_deserializer=lightning__pb2.GetRecoveryInfoRequest.FromString,
                    response_serializer=lightning__pb2.GetRecoveryInfoResponse.SerializeToString,
            ),
            'PendingChannels': grpc.unary_unary_rpc_method_handler(
                    servicer.PendingChannels,
                    request_deserializer=lightning__pb2.PendingChannelsRequest.FromString,
                    response_serializer=lightning__pb2.PendingChannelsResponse.SerializeToString,
            ),
            'ListChannels': grpc.unary_unary_rpc_method_handler(
                    servicer.ListChannels,
                    request_deserializer=lightning__pb2.ListChannelsRequest.FromString,
                    response_serializer=lightning__pb2.ListChannelsResponse.SerializeToString,
            ),
            'SubscribeChannelEvents': grpc.unary_stream_rpc_method_handler(
                    servicer.SubscribeChannelEvents,
                    request_deserializer=lightning__pb2.ChannelEventSubscription.FromString,
                    response_serializer=lightning__pb2.ChannelEventUpdate.SerializeToString,
            ),
            'ClosedChannels': grpc.unary_unary_rpc_method_handler(
                    servicer.ClosedChannels,
                    request_deserializer=lightning__pb2.ClosedChannelsRequest.FromString,
                    response_serializer=lightning__pb2.ClosedChannelsResponse.SerializeToString,
            ),
            'OpenChannelSync': grpc.unary_unary_rpc_method_handler(
                    servicer.OpenChannelSync,
                    request_deserializer=lightning__pb2.OpenChannelRequest.FromString,
                    response_serializer=lightning__pb2.ChannelPoint.SerializeToString,
            ),
            'OpenChannel': grpc.unary_stream_rpc_method_handler(
                    servicer.OpenChannel,
                    request_deserializer=lightning__pb2.OpenChannelRequest.FromString,
                    response_serializer=lightning__pb2.OpenStatusUpdate.SerializeToString,
            ),
            'BatchOpenChannel': grpc.unary_unary_rpc_method_handler(
                    servicer.BatchOpenChannel,
                    request_deserializer=lightning__pb2.BatchOpenChannelRequest.FromString,
                    response_serializer=lightning__pb2.BatchOpenChannelResponse.SerializeToString,
            ),
            'FundingStateStep': grpc.unary_unary_rpc_method_handler(
                    servicer.FundingStateStep,
                    request_deserializer=lightning__pb2.FundingTransitionMsg.FromString,
                    response_serializer=lightning__pb2.FundingStateStepResp.SerializeToString,
            ),
            'ChannelAcceptor': grpc.stream_stream_rpc_method_handler(
                    servicer.ChannelAcceptor,
                    request_deserializer=lightning__pb2.ChannelAcceptResponse.FromString,
                    response_serializer=lightning__pb2.ChannelAcceptRequest.SerializeToString,
            ),
            'CloseChannel': grpc.unary_stream_rpc_method_handler(
                    servicer.CloseChannel,
                    request_deserializer=lightning__pb2.CloseChannelRequest.FromString,
                    response_serializer=lightning__pb2.CloseStatusUpdate.SerializeToString,
            ),
            'AbandonChannel': grpc.unary_unary_rpc_method_handler(
                    servicer.AbandonChannel,
                    request_deserializer=lightning__pb2.AbandonChannelRequest.FromString,
                    response_serializer=lightning__pb2.AbandonChannelResponse.SerializeToString,
            ),
            'SendPayment': grpc.stream_stream_rpc_method_handler(
                    servicer.SendPayment,
                    request_deserializer=lightning__pb2.SendRequest.FromString,
                    response_serializer=lightning__pb2.SendResponse.SerializeToString,
            ),
            'SendPaymentSync': grpc.unary_unary_rpc_method_handler(
                    servicer.SendPaymentSync,
                    request_deserializer=lightning__pb2.SendRequest.FromString,
                    response_serializer=lightning__pb2.SendResponse.SerializeToString,
            ),
            'SendToRoute': grpc.stream_stream_rpc_method_handler(
                    servicer.SendToRoute,
                    request_deserializer=lightning__pb2.SendToRouteRequest.FromString,
                    response_serializer=lightning__pb2.SendResponse.SerializeToString,
            ),
            'SendToRouteSync': grpc.unary_unary_rpc_method_handler(
                    servicer.SendToRouteSync,
                    request_deserializer=lightning__pb2.SendToRouteRequest.FromString,
                    response_serializer=lightning__pb2.SendResponse.SerializeToString,
            ),
            'AddInvoice': grpc.unary_unary_rpc_method_handler(
                    servicer.AddInvoice,
                    request_deserializer=lightning__pb2.Invoice.FromString,
                    response_serializer=lightning__pb2.AddInvoiceResponse.SerializeToString,
            ),
            'ListInvoices': grpc.unary_unary_rpc_method_handler(
                    servicer.ListInvoices,
                    request_deserializer=lightning__pb2.ListInvoiceRequest.FromString,
                    response_serializer=lightning__pb2.ListInvoiceResponse.SerializeToString,
            ),
            'LookupInvoice': grpc.unary_unary_rpc_method_handler(
                    servicer.LookupInvoice,
                    request_deserializer=lightning__pb2.PaymentHash.FromString,
                    response_serializer=lightning__pb2.Invoice.SerializeToString,
            ),
            'SubscribeInvoices': grpc.unary_stream_rpc_method_handler(
                    servicer.SubscribeInvoices,
                    request_deserializer=lightning__pb2.InvoiceSubscription.FromString,
                    response_serializer=lightning__pb2.Invoice.SerializeToString,
            ),
            'DecodePayReq': grpc.unary_unary_rpc_method_handler(
                    servicer.DecodePayReq,
                    request_deserializer=lightning__pb2.PayReqString.FromString,
                    response_serializer=lightning__pb2.PayReq.SerializeToString,
            ),
            'ListPayments': grpc.unary_unary_rpc_method_handler(
                    servicer.ListPayments,
                    request_deserializer=lightning__pb2.ListPaymentsRequest.FromString,
                    response_serializer=lightning__pb2.ListPaymentsResponse.SerializeToString,
            ),
            'DeletePayment': grpc.unary_unary_rpc_method_handler(
                    servicer.DeletePayment,
                    request_deserializer=lightning__pb2.DeletePaymentRequest.FromString,
                    response_serializer=lightning__pb2.DeletePaymentResponse.SerializeToString,
            ),
            'DeleteAllPayments': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteAllPayments,
                    request_deserializer=lightning__pb2.DeleteAllPaymentsRequest.FromString,
                    response_serializer=lightning__pb2.DeleteAllPaymentsResponse.SerializeToString,
            ),
            'DescribeGraph': grpc.unary_unary_rpc_method_handler(
                    servicer.DescribeGraph,
                    request_deserializer=lightning__pb2.ChannelGraphRequest.FromString,
                    response_serializer=lightning__pb2.ChannelGraph.SerializeToString,
            ),
            'GetNodeMetrics': grpc.unary_unary_rpc_method_handler(
                    servicer.GetNodeMetrics,
                    request_deserializer=lightning__pb2.NodeMetricsRequest.FromString,
                    response_serializer=lightning__pb2.NodeMetricsResponse.SerializeToString,
            ),
            'GetChanInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetChanInfo,
                    request_deserializer=lightning__pb2.ChanInfoRequest.FromString,
                    response_serializer=lightning__pb2.ChannelEdge.SerializeToString,
            ),
            'GetNodeInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetNodeInfo,
                    request_deserializer=lightning__pb2.NodeInfoRequest.FromString,
                    response_serializer=lightning__pb2.NodeInfo.SerializeToString,
            ),
            'QueryRoutes': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryRoutes,
                    request_deserializer=lightning__pb2.QueryRoutesRequest.FromString,
                    response_serializer=lightning__pb2.QueryRoutesResponse.SerializeToString,
            ),
            'GetNetworkInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetNetworkInfo,
                    request_deserializer=lightning__pb2.NetworkInfoRequest.FromString,
                    response_serializer=lightning__pb2.NetworkInfo.SerializeToString,
            ),
            'StopDaemon': grpc.unary_unary_rpc_method_handler(
                    servicer.StopDaemon,
                    request_deserializer=lightning__pb2.StopRequest.FromString,
                    response_serializer=lightning__pb2.StopResponse.SerializeToString,
            ),
            'SubscribeChannelGraph': grpc.unary_stream_rpc_method_handler(
                    servicer.SubscribeChannelGraph,
                    request_deserializer=lightning__pb2.GraphTopologySubscription.FromString,
                    response_serializer=lightning__pb2.GraphTopologyUpdate.SerializeToString,
            ),
            'DebugLevel': grpc.unary_unary_rpc_method_handler(
                    servicer.DebugLevel,
                    request_deserializer=lightning__pb2.DebugLevelRequest.FromString,
                    response_serializer=lightning__pb2.DebugLevelResponse.SerializeToString,
            ),
            'FeeReport': grpc.unary_unary_rpc_method_handler(
                    servicer.FeeReport,
                    request_deserializer=lightning__pb2.FeeReportRequest.FromString,
                    response_serializer=lightning__pb2.FeeReportResponse.SerializeToString,
            ),
            'UpdateChannelPolicy': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateChannelPolicy,
                    request_deserializer=lightning__pb2.PolicyUpdateRequest.FromString,
                    response_serializer=lightning__pb2.PolicyUpdateResponse.SerializeToString,
            ),
            'ForwardingHistory': grpc.unary_unary_rpc_method_handler(
                    servicer.ForwardingHistory,
                    request_deserializer=lightning__pb2.ForwardingHistoryRequest.FromString,
                    response_serializer=lightning__pb2.ForwardingHistoryResponse.SerializeToString,
            ),
            'ExportChannelBackup': grpc.unary_unary_rpc_method_handler(
                    servicer.ExportChannelBackup,
                    request_deserializer=lightning__pb2.ExportChannelBackupRequest.FromString,
                    response_serializer=lightning__pb2.ChannelBackup.SerializeToString,
            ),
            'ExportAllChannelBackups': grpc.unary_unary_rpc_method_handler(
                    servicer.ExportAllChannelBackups,
                    request_deserializer=lightning__pb2.ChanBackupExportRequest.FromString,
                    response_serializer=lightning__pb2.ChanBackupSnapshot.SerializeToString,
            ),
            'VerifyChanBackup': grpc.unary_unary_rpc_method_handler(
                    servicer.VerifyChanBackup,
                    request_deserializer=lightning__pb2.ChanBackupSnapshot.FromString,
                    response_serializer=lightning__pb2.VerifyChanBackupResponse.SerializeToString,
            ),
            'RestoreChannelBackups': grpc.unary_unary_rpc_method_handler(
                    servicer.RestoreChannelBackups,
                    request_deserializer=lightning__pb2.RestoreChanBackupRequest.FromString,
                    response_serializer=lightning__pb2.RestoreBackupResponse.SerializeToString,
            ),
            'SubscribeChannelBackups': grpc.unary_stream_rpc_method_handler(
                    servicer.SubscribeChannelBackups,
                    request_deserializer=lightning__pb2.ChannelBackupSubscription.FromString,
                    response_serializer=lightning__pb2.ChanBackupSnapshot.SerializeToString,
            ),
            'BakeMacaroon': grpc.unary_unary_rpc_method_handler(
                    servicer.BakeMacaroon,
                    request_deserializer=lightning__pb2.BakeMacaroonRequest.FromString,
                    response_serializer=lightning__pb2.BakeMacaroonResponse.SerializeToString,
            ),
            'ListMacaroonIDs': grpc.unary_unary_rpc_method_handler(
                    servicer.ListMacaroonIDs,
                    request_deserializer=lightning__pb2.ListMacaroonIDsRequest.FromString,
                    response_serializer=lightning__pb2.ListMacaroonIDsResponse.SerializeToString,
            ),
            'DeleteMacaroonID': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteMacaroonID,
                    request_deserializer=lightning__pb2.DeleteMacaroonIDRequest.FromString,
                    response_serializer=lightning__pb2.DeleteMacaroonIDResponse.SerializeToString,
            ),
            'ListPermissions': grpc.unary_unary_rpc_method_handler(
                    servicer.ListPermissions,
                    request_deserializer=lightning__pb2.ListPermissionsRequest.FromString,
                    response_serializer=lightning__pb2.ListPermissionsResponse.SerializeToString,
            ),
            'CheckMacaroonPermissions': grpc.unary_unary_rpc_method_handler(
                    servicer.CheckMacaroonPermissions,
                    request_deserializer=lightning__pb2.CheckMacPermRequest.FromString,
                    response_serializer=lightning__pb2.CheckMacPermResponse.SerializeToString,
            ),
            'RegisterRPCMiddleware': grpc.stream_stream_rpc_method_handler(
                    servicer.RegisterRPCMiddleware,
                    request_deserializer=lightning__pb2.RPCMiddlewareResponse.FromString,
                    response_serializer=lightning__pb2.RPCMiddlewareRequest.SerializeToString,
            ),
            'SendCustomMessage': grpc.unary_unary_rpc_method_handler(
                    servicer.SendCustomMessage,
                    request_deserializer=lightning__pb2.SendCustomMessageRequest.FromString,
                    response_serializer=lightning__pb2.SendCustomMessageResponse.SerializeToString,
            ),
            'SubscribeCustomMessages': grpc.unary_stream_rpc_method_handler(
                    servicer.SubscribeCustomMessages,
                    request_deserializer=lightning__pb2.SubscribeCustomMessagesRequest.FromString,
                    response_serializer=lightning__pb2.CustomMessage.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'lnrpc.Lightning', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Lightning(object):
    """
    Comments in this file will be directly parsed into the API
    Documentation as descriptions of the associated method, message, or field.
    These descriptions should go right above the definition of the object, and
    can be in either block or // comment format.

    An RPC method can be matched to an lncli command by placing a line in the
    beginning of the description in exactly the following format:
    lncli: `methodname`

    Failure to specify the exact name of the command will cause documentation
    generation to fail.

    More information on how exactly the gRPC documentation is generated from
    this proto file can be found here:
    https://github.com/lightninglabs/lightning-api

    Lightning is the main RPC server of the daemon.
    """

    @staticmethod
    def WalletBalance(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/WalletBalance',
            lightning__pb2.WalletBalanceRequest.SerializeToString,
            lightning__pb2.WalletBalanceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ChannelBalance(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/ChannelBalance',
            lightning__pb2.ChannelBalanceRequest.SerializeToString,
            lightning__pb2.ChannelBalanceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTransactions(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/GetTransactions',
            lightning__pb2.GetTransactionsRequest.SerializeToString,
            lightning__pb2.TransactionDetails.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def EstimateFee(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/EstimateFee',
            lightning__pb2.EstimateFeeRequest.SerializeToString,
            lightning__pb2.EstimateFeeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SendCoins(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/SendCoins',
            lightning__pb2.SendCoinsRequest.SerializeToString,
            lightning__pb2.SendCoinsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListUnspent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/ListUnspent',
            lightning__pb2.ListUnspentRequest.SerializeToString,
            lightning__pb2.ListUnspentResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SubscribeTransactions(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/lnrpc.Lightning/SubscribeTransactions',
            lightning__pb2.GetTransactionsRequest.SerializeToString,
            lightning__pb2.Transaction.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SendMany(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/SendMany',
            lightning__pb2.SendManyRequest.SerializeToString,
            lightning__pb2.SendManyResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def NewAddress(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/NewAddress',
            lightning__pb2.NewAddressRequest.SerializeToString,
            lightning__pb2.NewAddressResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SignMessage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/SignMessage',
            lightning__pb2.SignMessageRequest.SerializeToString,
            lightning__pb2.SignMessageResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VerifyMessage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/VerifyMessage',
            lightning__pb2.VerifyMessageRequest.SerializeToString,
            lightning__pb2.VerifyMessageResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ConnectPeer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/ConnectPeer',
            lightning__pb2.ConnectPeerRequest.SerializeToString,
            lightning__pb2.ConnectPeerResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DisconnectPeer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/DisconnectPeer',
            lightning__pb2.DisconnectPeerRequest.SerializeToString,
            lightning__pb2.DisconnectPeerResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListPeers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/ListPeers',
            lightning__pb2.ListPeersRequest.SerializeToString,
            lightning__pb2.ListPeersResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SubscribePeerEvents(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/lnrpc.Lightning/SubscribePeerEvents',
            lightning__pb2.PeerEventSubscription.SerializeToString,
            lightning__pb2.PeerEvent.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/GetInfo',
            lightning__pb2.GetInfoRequest.SerializeToString,
            lightning__pb2.GetInfoResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRecoveryInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/GetRecoveryInfo',
            lightning__pb2.GetRecoveryInfoRequest.SerializeToString,
            lightning__pb2.GetRecoveryInfoResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def PendingChannels(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/PendingChannels',
            lightning__pb2.PendingChannelsRequest.SerializeToString,
            lightning__pb2.PendingChannelsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListChannels(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/ListChannels',
            lightning__pb2.ListChannelsRequest.SerializeToString,
            lightning__pb2.ListChannelsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SubscribeChannelEvents(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/lnrpc.Lightning/SubscribeChannelEvents',
            lightning__pb2.ChannelEventSubscription.SerializeToString,
            lightning__pb2.ChannelEventUpdate.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ClosedChannels(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/ClosedChannels',
            lightning__pb2.ClosedChannelsRequest.SerializeToString,
            lightning__pb2.ClosedChannelsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def OpenChannelSync(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/OpenChannelSync',
            lightning__pb2.OpenChannelRequest.SerializeToString,
            lightning__pb2.ChannelPoint.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def OpenChannel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/lnrpc.Lightning/OpenChannel',
            lightning__pb2.OpenChannelRequest.SerializeToString,
            lightning__pb2.OpenStatusUpdate.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def BatchOpenChannel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/BatchOpenChannel',
            lightning__pb2.BatchOpenChannelRequest.SerializeToString,
            lightning__pb2.BatchOpenChannelResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FundingStateStep(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/FundingStateStep',
            lightning__pb2.FundingTransitionMsg.SerializeToString,
            lightning__pb2.FundingStateStepResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ChannelAcceptor(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/lnrpc.Lightning/ChannelAcceptor',
            lightning__pb2.ChannelAcceptResponse.SerializeToString,
            lightning__pb2.ChannelAcceptRequest.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CloseChannel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/lnrpc.Lightning/CloseChannel',
            lightning__pb2.CloseChannelRequest.SerializeToString,
            lightning__pb2.CloseStatusUpdate.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AbandonChannel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/AbandonChannel',
            lightning__pb2.AbandonChannelRequest.SerializeToString,
            lightning__pb2.AbandonChannelResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SendPayment(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/lnrpc.Lightning/SendPayment',
            lightning__pb2.SendRequest.SerializeToString,
            lightning__pb2.SendResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SendPaymentSync(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/SendPaymentSync',
            lightning__pb2.SendRequest.SerializeToString,
            lightning__pb2.SendResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SendToRoute(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/lnrpc.Lightning/SendToRoute',
            lightning__pb2.SendToRouteRequest.SerializeToString,
            lightning__pb2.SendResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SendToRouteSync(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/SendToRouteSync',
            lightning__pb2.SendToRouteRequest.SerializeToString,
            lightning__pb2.SendResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddInvoice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/AddInvoice',
            lightning__pb2.Invoice.SerializeToString,
            lightning__pb2.AddInvoiceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListInvoices(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/ListInvoices',
            lightning__pb2.ListInvoiceRequest.SerializeToString,
            lightning__pb2.ListInvoiceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def LookupInvoice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/LookupInvoice',
            lightning__pb2.PaymentHash.SerializeToString,
            lightning__pb2.Invoice.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SubscribeInvoices(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/lnrpc.Lightning/SubscribeInvoices',
            lightning__pb2.InvoiceSubscription.SerializeToString,
            lightning__pb2.Invoice.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DecodePayReq(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/DecodePayReq',
            lightning__pb2.PayReqString.SerializeToString,
            lightning__pb2.PayReq.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListPayments(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/ListPayments',
            lightning__pb2.ListPaymentsRequest.SerializeToString,
            lightning__pb2.ListPaymentsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeletePayment(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/DeletePayment',
            lightning__pb2.DeletePaymentRequest.SerializeToString,
            lightning__pb2.DeletePaymentResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteAllPayments(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/DeleteAllPayments',
            lightning__pb2.DeleteAllPaymentsRequest.SerializeToString,
            lightning__pb2.DeleteAllPaymentsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DescribeGraph(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/DescribeGraph',
            lightning__pb2.ChannelGraphRequest.SerializeToString,
            lightning__pb2.ChannelGraph.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetNodeMetrics(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/GetNodeMetrics',
            lightning__pb2.NodeMetricsRequest.SerializeToString,
            lightning__pb2.NodeMetricsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetChanInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/GetChanInfo',
            lightning__pb2.ChanInfoRequest.SerializeToString,
            lightning__pb2.ChannelEdge.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetNodeInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/GetNodeInfo',
            lightning__pb2.NodeInfoRequest.SerializeToString,
            lightning__pb2.NodeInfo.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryRoutes(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/QueryRoutes',
            lightning__pb2.QueryRoutesRequest.SerializeToString,
            lightning__pb2.QueryRoutesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetNetworkInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/GetNetworkInfo',
            lightning__pb2.NetworkInfoRequest.SerializeToString,
            lightning__pb2.NetworkInfo.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StopDaemon(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/StopDaemon',
            lightning__pb2.StopRequest.SerializeToString,
            lightning__pb2.StopResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SubscribeChannelGraph(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/lnrpc.Lightning/SubscribeChannelGraph',
            lightning__pb2.GraphTopologySubscription.SerializeToString,
            lightning__pb2.GraphTopologyUpdate.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DebugLevel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/DebugLevel',
            lightning__pb2.DebugLevelRequest.SerializeToString,
            lightning__pb2.DebugLevelResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FeeReport(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/FeeReport',
            lightning__pb2.FeeReportRequest.SerializeToString,
            lightning__pb2.FeeReportResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateChannelPolicy(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/UpdateChannelPolicy',
            lightning__pb2.PolicyUpdateRequest.SerializeToString,
            lightning__pb2.PolicyUpdateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ForwardingHistory(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/ForwardingHistory',
            lightning__pb2.ForwardingHistoryRequest.SerializeToString,
            lightning__pb2.ForwardingHistoryResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ExportChannelBackup(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/ExportChannelBackup',
            lightning__pb2.ExportChannelBackupRequest.SerializeToString,
            lightning__pb2.ChannelBackup.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ExportAllChannelBackups(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/ExportAllChannelBackups',
            lightning__pb2.ChanBackupExportRequest.SerializeToString,
            lightning__pb2.ChanBackupSnapshot.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VerifyChanBackup(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/VerifyChanBackup',
            lightning__pb2.ChanBackupSnapshot.SerializeToString,
            lightning__pb2.VerifyChanBackupResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RestoreChannelBackups(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/RestoreChannelBackups',
            lightning__pb2.RestoreChanBackupRequest.SerializeToString,
            lightning__pb2.RestoreBackupResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SubscribeChannelBackups(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/lnrpc.Lightning/SubscribeChannelBackups',
            lightning__pb2.ChannelBackupSubscription.SerializeToString,
            lightning__pb2.ChanBackupSnapshot.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def BakeMacaroon(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/BakeMacaroon',
            lightning__pb2.BakeMacaroonRequest.SerializeToString,
            lightning__pb2.BakeMacaroonResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListMacaroonIDs(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/ListMacaroonIDs',
            lightning__pb2.ListMacaroonIDsRequest.SerializeToString,
            lightning__pb2.ListMacaroonIDsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteMacaroonID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/DeleteMacaroonID',
            lightning__pb2.DeleteMacaroonIDRequest.SerializeToString,
            lightning__pb2.DeleteMacaroonIDResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListPermissions(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/ListPermissions',
            lightning__pb2.ListPermissionsRequest.SerializeToString,
            lightning__pb2.ListPermissionsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CheckMacaroonPermissions(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/CheckMacaroonPermissions',
            lightning__pb2.CheckMacPermRequest.SerializeToString,
            lightning__pb2.CheckMacPermResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RegisterRPCMiddleware(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/lnrpc.Lightning/RegisterRPCMiddleware',
            lightning__pb2.RPCMiddlewareResponse.SerializeToString,
            lightning__pb2.RPCMiddlewareRequest.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SendCustomMessage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lnrpc.Lightning/SendCustomMessage',
            lightning__pb2.SendCustomMessageRequest.SerializeToString,
            lightning__pb2.SendCustomMessageResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SubscribeCustomMessages(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/lnrpc.Lightning/SubscribeCustomMessages',
            lightning__pb2.SubscribeCustomMessagesRequest.SerializeToString,
            lightning__pb2.CustomMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



"""Module for channel rebalancing."""
import logging
import math
from typing import TYPE_CHECKING, Optional, Dict
import time

from lndmanage.lib.rating import node_badness
from lndmanage.lib.routing import Router
from lndmanage.lib import exceptions
from lndmanage.lib.forwardings import get_channel_properties
from lndmanage.lib.exceptions import (
    DryRun,
    NoRebalanceCandidates,
    NoRoute,
    NotEconomic,
    PaymentTimeOut,
    RebalanceFailure,
    RebalancingTrialsExhausted,
    TooExpensive,
)
from lndmanage.lib.ln_utilities import unbalancedness_to_local_balance, parse_nodeid_channelid
from lndmanage import settings

if TYPE_CHECKING:
    from lndmanage.lib.node import LndNode

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())

DEFAULT_MAX_FEE_RATE = 0.001000
DEFAULT_AMOUNT_SAT = 100000
RESERVED_REBALANCE_FEE_RATE_MILLI_MSAT = 50  # a buffer for the fee rate a rebalance route can cost
FORWARDING_STATS_DAYS = 30  # how many days will be taken into account when determining the rebalance direction
MIN_REBALANCE_AMOUNT_SAT = 20_000
MAX_UNBALANCEDNESS_FOR_CANDIDATES = 0.2  # sending rebalance candidates will not have an unbalancedness higher than this


class Rebalancer(object):
    """Implements methods for rebalancing."""

    def __init__(self, node: 'LndNode', max_effective_fee_rate: float, budget_sat: int, force=False):
        """
        :param node: node instance
        :param max_effective_fee_rate: maximum effective fee rate (base_fee + fee_rate * amt)/amt paid
        :param budget_sat: maximal rebalancing budget
        :param force: allow uneconomic routes / rebalance candidates
        """
        self.node = node
        self.channels = {}
        self.router = Router(self.node)
        # we don't want to route over our node, so blacklist it:
        self.node.network.channel_rater.blacklist_add_node(self.node.pub_key)
        self.max_effective_fee_rate = max_effective_fee_rate
        self.budget_sat = budget_sat
        self.force = force

    def _rebalance(
            self,
            send_channels: Dict[int, dict],
            receive_channels: Dict[int, dict],
            amt_sat: int,
            payment_hash: bytes,
            payment_address: bytes,
            budget_sat: int,
            dry=False,
    ) -> int:
        """Rebalances liquidity from send_channels to receive_channels with an amount of
        amt_sat.

        A prior created payment_hash has to be given. The budget_sat sets
        the maxmimum fees in sat that will be paid. A dry run can be done.

        :param send_channels: channels for sending with info
        :param receive_channels: channels for receiving with info
        :param amt_sat: amount to be sent in sat
        :param payment_hash: payment hash
        :param payment_address: payment secret
        :param budget_sat: budget for the rebalance in sat
        :param dry: specifies, if it is a dry run

        :return: total fees for the whole rebalance in msat

        :raises TemporarayChannelFailure: no liquidity along path
        :raises UnknownNextPeer: a routing node didn't where to forward
        :raises DuplicateRoute: the same route was already tried
        :raises NoRoute: no route was found from source to destination
        :raises TooExpensive: the circular payment would exceed the fee limits
        :raises PaymentTimeOut: the payment timed out
        :raises DryRun: attempt was just a dry run
        :raises NotEconomic: we would effectively loose money due to not enough expected
            earnings in the future
        """
        # be up to date with the blockheight, otherwise could lead to cltv errors
        self.node.update_blockheight()
        amt_msat = amt_sat * 1000

        count = 0
        while True:
            start_time = time.time()
            count += 1
            if count > settings.REBALANCING_TRIALS:
                raise RebalancingTrialsExhausted
            logger.info(f">>> Trying to rebalance with {amt_sat} sat (attempt number {count}).")

            route = self.router.get_route(send_channels, receive_channels, amt_msat)
            if not route:
                raise NoRoute

            effective_fee_rate = route.total_fee_msat / route.total_amt_msat
            logger.info(
                f"  > Route summary: amount: {(route.total_amt_msat - route.total_fee_msat) / 1000:3.3f} "
                f"sat, total fee: {route.total_fee_msat / 1000:3.3f} sat, "
                f"fee rate: {effective_fee_rate:1.6f}, "
                f"number of hops: {len(route.channel_hops)}")
            logger.debug(f"   Channel hops: {route.channel_hops}")

            # check if route makes sense
            illiquid_channel_id = route.channel_hops[-1]
            illiquid_channel = self.channels[illiquid_channel_id]
            liquid_channel_id = route.channel_hops[0]
            liquid_channel = self.channels[liquid_channel_id]
            assert illiquid_channel_id in list(receive_channels.keys()), "receiving channel should be in receive list"
            assert liquid_channel_id in list(send_channels.keys()), "sending channel should be in send list"

            # check economics
            fee_rate_margin = (illiquid_channel['local_fee_rate'] - liquid_channel['local_fee_rate']) / 1_000_000
            logger.info(f"  > Expected gain: {(fee_rate_margin - effective_fee_rate) * amt_sat:3.3f} sat")
            if (effective_fee_rate > fee_rate_margin) and not self.force:
                # TODO: We could look for the hop that charges the highest fee
                #  and blacklist it, to ignore it in the next path.
                pass

            if effective_fee_rate > self.max_effective_fee_rate:
                raise TooExpensive(f"Route is too expensive (rate too high). Rate: {effective_fee_rate:.6f}, "
                                   f"requested max rate: {self.max_effective_fee_rate:.6f}")

            if route.total_fee_msat > budget_sat * 1000:
                raise TooExpensive(f"Route is too expensive (budget exhausted). Total fee of route: "
                                   f"{route.total_fee_msat / 1000:.3f} sat, budget: {budget_sat:.3f} sat")

            def report_success_up_to_failed_hop(failed_hop_index: Optional[int]):
                """Rates the route."""
                end_time = time.time()
                elapsed_time = end_time - start_time
                logger.debug(f"  > time elapsed: {elapsed_time:3.1f} s")

                success_path_length = failed_hop_index + 1 if failed_hop_index else len(route.hops)
                for hop, channel in enumerate(route.hops):
                    source_node = route.node_hops[hop]
                    target_node = route.node_hops[hop + 1]
                    self.node.network.liquidity_hints.update_elapsed_time(source_node, elapsed_time / success_path_length)
                    if failed_hop_index and hop == failed_hop_index:
                        break
                    self.node.network.liquidity_hints.update_can_send(source_node, target_node, channel['chan_id'], amt_msat)

                # symmetrically penalize a route about the error source if it failed
                if failed_hop_index:
                    for node_number, node in enumerate(route.node_hops):
                        badness = node_badness(node_number, failed_hop_index)
                        self.node.network.liquidity_hints.update_badness_hint(node, badness)

            if not dry:
                try:
                    result = self.node.send_to_route(route, payment_hash, payment_address)
                except PaymentTimeOut:
                    raise PaymentTimeOut
                # TODO: check whether the failure source is correctly attributed
                except exceptions.TemporaryChannelFailure as e:
                    failed_hop = int(e.payment.failure.failure_source_index)
                except exceptions.TemporaryNodeFailure as e:
                    failed_hop = int(e.payment.failure.failure_source_index)
                except exceptions.UnknownNextPeer as e:
                    failed_hop = int(e.payment.failure.failure_source_index)
                except exceptions.ChannelDisabled as e:
                    failed_hop = int(e.payment.failure.failure_source_index)
                except exceptions.FeeInsufficient as e:
                    failed_hop = int(e.payment.failure.failure_source_index)
                except exceptions.IncorrectCLTVExpiry as e:
                    failed_hop = int(e.payment.failure.failure_source_index)
                except exceptions.In as e:
                    failed_hop = int(e.payment.failure.failure_source_index)
                else:
                    logger.debug(f"Preimage: {result.preimage.hex()}")
                    logger.info("Success!\n")
                    report_success_up_to_failed_hop(failed_hop_index=None)
                    self.node.network.save_liquidty_hints()
                    return route.total_fee_msat

                if failed_hop:
                    failed_channel_id = route.hops[failed_hop]['chan_id']
                    failed_source = route.node_hops[failed_hop]
                    failed_target = route.node_hops[failed_hop + 1]

                    if failed_channel_id in [send_channels, receive_channels]:
                        raise RebalanceFailure(
                            f"Own channel failed. Something is wrong. "
                            f"This is likely due to a wrong accounting "
                            f"for the channel reserve and will be fixed "
                            f"in the future. "
                            f"Try with smaller absolute target. "
                            f"Failing channel: {failed_channel_id}")

                    # determine the nodes involved in the channel
                    logger.info(f"  > Failed: hop: {failed_hop + 1}, channel: {failed_channel_id}")
                    logger.info(f"  > Could not reach {failed_target} ({self.node.network.node_alias(failed_target)})\n")
                    logger.debug(f"  > Node hops {route.node_hops}")
                    logger.debug(f"  > Channel hops {route.channel_hops}")

                    # report that channel could not route the amount to liquidity hints
                    self.node.network.liquidity_hints.update_cannot_send(
                        failed_source, failed_target, failed_channel_id, amt_msat)

                    # report all the previous hops that they could route the amount
                    report_success_up_to_failed_hop(failed_hop)
                    self.node.network.save_liquidty_hints()
            else:
                raise DryRun

    def _get_rebalance_candidates(
            self,
            channel_id: int,
            channel_fee_rate_milli_msat: int,
            local_balance_change: int,
    ) -> Dict:
        """
        Determines channels, which can be used to rebalance a channel.

        If the local_balance_change is negative, the local balance of the to
        be balanced channel is tried to be reduced. This method determines
        channels with which we can rebalance by considering channels which have enough
        funds and for which rebalancing makes economically sense based on fee rates.

        :param channel_id: the channel id of the to be rebalanced channel
        :param channel_fee_rate_milli_msat: the local fee rate of the to be rebalanced
            channel
        :param local_balance_change: amount by which the local balance of
            channel should change in sat

        :return: rebalance candidates with information
        """
        # update the channel list to reflect up-to-date balances
        self.channels = self.node.get_unbalanced_channels()

        # need to make sure we don't rebalance with the same channel or other channels
        # of the same node
        map_channel_id_node_id = self.node.channel_id_to_node_id()
        rebalance_node_id = map_channel_id_node_id[channel_id]
        removed_channels = [cid for cid, nid in map_channel_id_node_id.items() if nid == rebalance_node_id]
        rebalance_candidates = {
            k: c for k, c in self.channels.items() if k not in removed_channels}

        # filter channels, which can't receive/send the amount
        candidates_send = True if local_balance_change > 0 else False
        rebalance_candidates_with_funds = {}
        for k, c in rebalance_candidates.items():
            if candidates_send:
                maximal_can_send = self._maximal_local_balance_change(False, c)
                if maximal_can_send > abs(local_balance_change) and \
                        c['unbalancedness'] < MAX_UNBALANCEDNESS_FOR_CANDIDATES:
                    rebalance_candidates_with_funds[k] = c
            else:
                maximal_can_receive = self._maximal_local_balance_change(True, c)
                if maximal_can_receive > abs(local_balance_change) and \
                        c['unbalancedness'] > -MAX_UNBALANCEDNESS_FOR_CANDIDATES:
                    rebalance_candidates_with_funds[k] = c

        # We only include rebalance candidates for which it makes economically sense to
        # rebalance. This is determined by the difference in fee rates:
        # For an increase in the local balance of a channel, we need to decrease the
        # local balance in another one:
        #   the fee rate margin is:
        #   fee_rate[rebalance_channel] - fee_rate[candidate_channel],
        #   it always needs to be positive at least
        rebalance_candidates_filtered = {}
        for k, c in rebalance_candidates_with_funds.items():
            if not self.force:  # we allow only economic candidates
                if local_balance_change < 0:  # we take liquidity out of the channel
                    fee_rate_margin = c['local_fee_rate'] - channel_fee_rate_milli_msat
                else:  # putting liquidity into the channel
                    fee_rate_margin = channel_fee_rate_milli_msat - c['local_fee_rate']
                # We enforce a mimimum amount of an acceptable fee rate,
                # because we need to also pay for rebalancing.
                if fee_rate_margin > RESERVED_REBALANCE_FEE_RATE_MILLI_MSAT:
                    c['fee_rate_margin'] = fee_rate_margin / 1_000_000
                    rebalance_candidates_filtered[k] = c
            else:  # otherwise, we can afford very high fee rates
                c['fee_rate_margin'] = float('inf')
                rebalance_candidates_filtered[k] = c
        return rebalance_candidates_filtered

    @staticmethod
    def _effective_fee_rate(amt_sat: int, base_fee: float, fee_rate: float) -> float:
        """
        Calculates the effective fee rate: (base_fee + fee_rate * amt) / amt

        :param amt_sat: amount in sat
        :param base_fee: base fee in sat
        :param fee_rate: fee rate

        :return: effective fee rate
        """
        amt_msat = amt_sat * 1000
        assert not (amt_msat == 0)
        fee_rate = (base_fee + fee_rate * amt_msat / 1000000) / amt_msat
        return fee_rate

    @staticmethod
    def _debug_rebalance_candidates(rebalance_candidates: Dict[int, dict]):
        """
        Prints rebalance candidates.

        :param rebalance_candidates:
        """
        logger.debug(f"-------- Description --------")
        logger.debug(
            "cid: channel id\n"
            "ub: unbalancedness (see --help)\n"
            "atb: amount to be balanced [sat]\n"
            "aaf: amount affordable [sat]\n"
            "l: local balance [sat]\n"
            "r: remote balance [sat]\n"
            "lbf: local base fee [msat]\n"
            "lfr: local fee rate\n"
            "frm: fee rate margin\n"
            "a: alias"
        )

        logger.debug(f"-------- Candidates in order of rebalance attempts "
                     f"--------")
        for c in rebalance_candidates.values():
            logger.debug(
                f"cid:{c['chan_id']} "
                f"ub:{c['unbalancedness']: 4.2f} "
                f"l:{c['local_balance']: 9d} "
                f"r:{c['remote_balance']: 9d} "
                f"lbf:{c['local_base_fee']: 6d} "
                f"lfr:{c['local_fee_rate']/1E6: 1.6f} "
                f"fra:{c.get('fee_rate_margin'): 1.6f} "
                f"a:{c['alias']}")

    @staticmethod
    def _maximal_local_balance_change(
            increase_local_balance: bool,
            unbalanced_channel_info: dict
    ) -> int:
        """Finds the amount to maximally send/receive via the channel."""
        local_balance = unbalanced_channel_info['local_balance']
        remote_balance = unbalanced_channel_info['remote_balance']
        remote_channel_reserve = unbalanced_channel_info['remote_chan_reserve_sat']
        local_channel_reserve = unbalanced_channel_info['local_chan_reserve_sat']

        if increase_local_balance:  # we want to add funds to the channel
            local_balance_change = remote_balance
            local_balance_change -= remote_channel_reserve
        else:  # we want to decrease funds
            local_balance_change = local_balance
            local_balance_change -= local_channel_reserve

        # the local balance already reflects commitment transaction fees
        # in principle we should account for the HTLC output here
        local_balance_change -= 172 * unbalanced_channel_info['fee_per_kw'] / 1000

        # TODO: buffer for all the rest of costs which is why
        #  local_balance_change can be negative
        return max(0, int(local_balance_change))

    def _node_is_multiple_connected(self, pub_key: str) -> bool:
        """Checks if the node is connected to us via several channels.

        :param pub_key: node public key

        :return: true if number of channels to the node is larger than 1
        """
        n_channels = 0
        for pk, chan_info in self.channels.items():
            if chan_info['remote_pubkey'] == pub_key:
                n_channels += 1
        if n_channels > 1:
            return True
        else:
            return False

    def rebalance(
            self,
            node_id_channel_id: str,
            dry=False,
            target: float = None,
            amount_sat: int = None,
    ) -> int:
        """Automatically rebalances a selected channel with a fee cap of
        self.budget_sat and self.max_effective_fee_rate.

        Rebalancing candidates are selected among all channels that support the
        rebalancing operation (liquidity-wise), which are economically viable
        (controlled through self.force) determined by fees rates of
        counterparty channels.

        The rebalancing operation is carried out in these steps:
        0. determine balancing amount, determine rebalance direction
        1. determine counterparty channels for the balancing amount
        2. try to rebalance with the cheapest route (taking into account different metrics including fees)
        3. if it doesn't work for several attempts, go to 1. with a reduced amount

        :param node_id_channel_id: the id of the peer or channel to be rebalanced
        :param dry: if set, it's a dry run
        :param target: specifies unbalancedness after rebalancing in [-1, 1]
        :param amount_sat: rebalance amount (target takes precedence)

        :return: fees in msat paid for rebalancing

        :raises MultichannelInboundRebalanceFailure: can't rebalance channels
            with a node we are mutiply connected
        :raises NoRebalanceCandidates: there are no counterparty rebalance
            candidates
        :raises RebalanceCandidatesExhausted: no more conterparty rebalance
            candidates
        :raises TooExpensive: the rebalance became too expensive
        """
        if target and not (-1.0 <= target <= 1.0):
            raise ValueError("Target must be between -1.0 and 1.0.")

        # convert the node id to a channel id if possible
        self.channels = self.node.get_unbalanced_channels()
        channel_id, node_id = parse_nodeid_channelid(node_id_channel_id)
        if node_id:
            node_id_to_channel_ids_map = self.node.node_id_to_channel_ids()
            for nid, cs in node_id_to_channel_ids_map.items():
                if nid == node_id:
                    if len(cs) > 1:
                        raise ValueError("Several channels correspond to node id, please specify channel id.")
                    channel_id = cs[0]
        try:
            unbalanced_channel_info = self.channels[channel_id]
        except KeyError:
            raise RebalanceFailure("Channel not known or inactive.")

        # 0. determine the amount we want to send/receive on the channel
        if target is not None:
            if target < unbalanced_channel_info['unbalancedness']:
                increase_local_balance = True
            else:
                increase_local_balance = False

            maximal_abs_local_balance_change = self._maximal_local_balance_change(
                increase_local_balance, unbalanced_channel_info)
            target_local_balance, _ = unbalancedness_to_local_balance(
                target,
                unbalanced_channel_info['capacity'],
                unbalanced_channel_info['commit_fee'],
                unbalanced_channel_info['initiator']
            )
            abs_local_balance_change = abs(target_local_balance - unbalanced_channel_info['local_balance'])
            initial_local_balance_change = min(abs_local_balance_change, maximal_abs_local_balance_change)
            # encode the sign to send (< 0) or receive (> 0)
            initial_local_balance_change *= 1 if increase_local_balance else -1

            if abs(initial_local_balance_change) <= 10_000:
                logger.info(f"Channel already balanced.")
                return 0
        elif not amount_sat:  # if no target is given, we enforce some default amount
            now_sec = time.time()
            then_sec = now_sec - FORWARDING_STATS_DAYS * 24 * 3600
            forwarding_properties = get_channel_properties(self.node, then_sec, now_sec)
            channel_properties = forwarding_properties.get(channel_id)
            fees_in = channel_properties['fees_in']
            fees_out = channel_properties['fees_out']
            ub = unbalanced_channel_info['unbalancedness']
            flow = channel_properties['flow_direction']

            if abs(ub) > 0.95:  # there's no other option
                initial_local_balance_change = int(math.copysign(1, ub) * DEFAULT_AMOUNT_SAT)
                logger.debug("Default amount due to strong unbalancedness.")
            elif fees_in or fees_out:  # based on type of earnings
                if fees_out > fees_in:  # then we want to increase balance in channel
                    initial_local_balance_change = DEFAULT_AMOUNT_SAT
                else:
                    initial_local_balance_change = -DEFAULT_AMOUNT_SAT
                logger.debug("Default amount due to fees.")
            elif not math.isnan(flow):  # counter the flow, probably not executed
                initial_local_balance_change = int(math.copysign(1, flow) * DEFAULT_AMOUNT_SAT)
                logger.debug("Default amount due to flow.")
            else:  # fall back to unbalancedness
                initial_local_balance_change = int(math.copysign(1, ub) * DEFAULT_AMOUNT_SAT)
                logger.debug("Default amount due to unbalancedness.")
        else:  # based on manual amount, checking bounds
            increase_local_balance = True if amount_sat > 0 else False
            maximal_change = self._maximal_local_balance_change(
                increase_local_balance=increase_local_balance,
                unbalanced_channel_info=unbalanced_channel_info
            )
            if abs(amount_sat) > maximal_change:
                raise ValueError(
                    f"Channel cannot {'receive' if increase_local_balance else 'send'} "
                    f"(maximal value: {int(math.copysign(1, amount_sat) * maximal_change)} sat)."
                    f" lb: {unbalanced_channel_info['local_balance']} sat"
                    f" rb: {unbalanced_channel_info['remote_balance']} sat")
            initial_local_balance_change = amount_sat

        # determine budget and fee rate from local balance change:
        # budget fee_rate  result
        #    x      0      set fee rate
        #    0      x      set budget
        #    x      x      max(budget, budget from fee rate): set fee rate and budget
        #    0      0      set defaults from a fee rate
        net_change = abs(initial_local_balance_change)
        if self.budget_sat and not self.max_effective_fee_rate:
            self.max_effective_fee_rate = self.budget_sat / net_change
        elif not self.budget_sat and self.max_effective_fee_rate:
            self.budget_sat = int(self.max_effective_fee_rate * net_change)
        elif self.budget_sat and self.max_effective_fee_rate:
            budget_from_fee_rate = int(net_change * self.max_effective_fee_rate)
            budget = max(budget_from_fee_rate, self.budget_sat)
            self.budget_sat = budget
            self.max_effective_fee_rate = budget / net_change
        else:
            self.budget_sat = int(DEFAULT_MAX_FEE_RATE * net_change)
            self.max_effective_fee_rate = DEFAULT_MAX_FEE_RATE

        logger.info(f">>> Trying to rebalance channel {channel_id} "
                    f"with a max rate of {self.max_effective_fee_rate} "
                    f"and a max fee of {self.budget_sat} sat.")

        if dry:
            logger.info(f">>> This is a dry run, nothing to fear.")

        # copy the original target for bookkeeping
        local_balance_change_left = initial_local_balance_change

        logger.info(
            f">>> The channel status before rebalancing is "
            f"lb:{unbalanced_channel_info['local_balance']} sat "
            f"rb:{unbalanced_channel_info['remote_balance']} sat "
            f"cap:{unbalanced_channel_info['capacity']} sat.")
        logger.debug(
            f">>> Commit fee {unbalanced_channel_info['commit_fee']} "
            f"sat. We opened channel: {unbalanced_channel_info['initiator']}. "
            f"Channel reserve: "
            f"{unbalanced_channel_info['local_chan_reserve_sat']} sat.")
        logger.debug(
            f">>> The change in local balance towards the requested "
            f"target (ub={target if target else 0.0:3.2f})"
            f" is {initial_local_balance_change} sat.")

        # the fee rate that is charged by the to-be-rebalanced channel
        fee_rate_milli_msat = unbalanced_channel_info['local_fee_rate']

        # a channel reserve is only accounted for, if we opened the channel
        reserve_sat = 0 if not unbalanced_channel_info['initiator'] else unbalanced_channel_info['local_chan_reserve_sat']

        expected_target = -2 * ((
            unbalanced_channel_info['local_balance'] +
            initial_local_balance_change + reserve_sat) /
            float(unbalanced_channel_info['capacity'])) + 1

        info_str = f">>> Trying to change the local balance by " \
                   f"{initial_local_balance_change} sat.\n" \
                   f"    The expected unbalancedness target is " \
                   f"{expected_target:3.2f} (respecting channel reserve)"
        if target is not None:
            info_str += f", requested target is {target:3.2f}."
        logger.info(info_str)

        node_is_multiple_connected = self._node_is_multiple_connected(
            unbalanced_channel_info['remote_pubkey'])

        if initial_local_balance_change > 0 and node_is_multiple_connected:
            logger.info(
                ">>> Note: You try to send liquidity to a channel of a node you are\n"
                "    connected to with multiple channels. We cannot control over which\n"
                "    channel the funds are sent back to us due to so-called non-strict\n"
                "    forwarding, see:\n"
                "    https://github.com/lightningnetwork/lightning-rfc/blob/master/"
                "04-onion-routing.md#non-strict-forwarding.\n")

        logger.info(f">>> Rebalancing can take some time. Please be patient!\n")

        total_fees_paid_msat = 0
        # amount_sat is the amount we try to rebalance with, which is gradually reduced
        # to improve in rebalance success probability
        amount_sat = local_balance_change_left
        budget_sat = self.budget_sat

        # we try to rebalance with amount_sat until we reach the desired total local
        # balance change
        while abs(local_balance_change_left) >= 0:
            # 1. determine counterparty rebalance candidates
            rebalance_candidates = self._get_rebalance_candidates(
                channel_id, fee_rate_milli_msat, amount_sat)

            if len(rebalance_candidates) == 0:
                raise NoRebalanceCandidates(
                    "Didn't find counterparty rebalance candidates.")

            self._debug_rebalance_candidates(rebalance_candidates)

            if total_fees_paid_msat >= self.budget_sat * 1000:
                raise TooExpensive(
                    f"Fee budget exhausted. "
                    f"Total fees {total_fees_paid_msat / 1000:.3f} sat.")

            logger.info(
                f">>> Need to still change the local balance by "
                f"{local_balance_change_left} sat to reach the goal "
                f"of {initial_local_balance_change} sat. "
                f"Fees paid up to now: {total_fees_paid_msat / 1000:.3f} sat.")

            # for each rebalance amount, get a new invoice
            invoice = self.node.get_invoice(
                amt_msat=abs(amount_sat) * 1000,
                memo=f"lndmanage: Rebalance of channel {channel_id}.")
            payment_hash, payment_address = invoice.r_hash, invoice.payment_addr

            # set sending and receiving channels
            if amount_sat < 0:  # we send over the channel
                send_channels = {channel_id: unbalanced_channel_info}
                receive_channels = rebalance_candidates
            else:  # we receive over the channel
                send_channels = rebalance_candidates
                # TODO: we could also extend the receive channels to all channels with the node
                receive_channels = {channel_id: unbalanced_channel_info}

            # attempt the rebalance
            try:
                rebalance_fees_msat = self._rebalance(
                    send_channels, receive_channels, abs(amount_sat), payment_hash,
                    payment_address, budget_sat, dry=dry)

                # account for running costs / target
                budget_sat -= rebalance_fees_msat // 1000
                total_fees_paid_msat += rebalance_fees_msat
                local_balance_change_left -= amount_sat

                relative_amt_to_go = (local_balance_change_left /
                                      initial_local_balance_change)

                # if we succeeded to rebalance, we start again at a higher amount
                amount_sat = local_balance_change_left

                # perfect rebalancing is not always possible,
                # so terminate if at least 90% of amount was reached
                if relative_amt_to_go <= 0.10:
                    logger.info(
                        f"Goal is reached. Rebalancing done. "
                        f"Total fees were {total_fees_paid_msat / 1000:.3f} sat.")
                    return total_fees_paid_msat
            except DryRun:
                logger.info(
                    "Would have tried this route now, but it was a dry run.\n")
                return 0
            except (RebalancingTrialsExhausted, NoRoute, TooExpensive) as e:
                logger.info(e)
                # We have attempted to rebalance a lot of times or didn't find a route
                # with the current amount. To improve the success rate, we split the
                # amount.
                amount_sat //= 2
                if abs(amount_sat) < MIN_REBALANCE_AMOUNT_SAT:
                    raise RebalanceFailure(
                        "It is unlikely we can rebalance the channel. Attempts with "
                        "small amounts already failed.\n")
                logger.info(
                    f"Could not rebalance with this amount. Decreasing amount to "
                    f"{amount_sat} sat.\n"
                )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from lndmanage.grpc_compiled import signer_pb2 as signer__pb2


class SignerStub(object):
    """Signer is a service that gives access to the signing functionality of the
    daemon's wallet.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SignOutputRaw = channel.unary_unary(
                '/signrpc.Signer/SignOutputRaw',
                request_serializer=signer__pb2.SignReq.SerializeToString,
                response_deserializer=signer__pb2.SignResp.FromString,
                )
        self.ComputeInputScript = channel.unary_unary(
                '/signrpc.Signer/ComputeInputScript',
                request_serializer=signer__pb2.SignReq.SerializeToString,
                response_deserializer=signer__pb2.InputScriptResp.FromString,
                )
        self.SignMessage = channel.unary_unary(
                '/signrpc.Signer/SignMessage',
                request_serializer=signer__pb2.SignMessageReq.SerializeToString,
                response_deserializer=signer__pb2.SignMessageResp.FromString,
                )
        self.VerifyMessage = channel.unary_unary(
                '/signrpc.Signer/VerifyMessage',
                request_serializer=signer__pb2.VerifyMessageReq.SerializeToString,
                response_deserializer=signer__pb2.VerifyMessageResp.FromString,
                )
        self.DeriveSharedKey = channel.unary_unary(
                '/signrpc.Signer/DeriveSharedKey',
                request_serializer=signer__pb2.SharedKeyRequest.SerializeToString,
                response_deserializer=signer__pb2.SharedKeyResponse.FromString,
                )


class SignerServicer(object):
    """Signer is a service that gives access to the signing functionality of the
    daemon's wallet.
    """

    def SignOutputRaw(self, request, context):
        """
        SignOutputRaw is a method that can be used to generated a signature for a
        set of inputs/outputs to a transaction. Each request specifies details
        concerning how the outputs should be signed, which keys they should be
        signed with, and also any optional tweaks. The return value is a fixed
        64-byte signature (the same format as we use on the wire in Lightning).

        If we are  unable to sign using the specified keys, then an error will be
        returned.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ComputeInputScript(self, request, context):
        """
        ComputeInputScript generates a complete InputIndex for the passed
        transaction with the signature as defined within the passed SignDescriptor.
        This method should be capable of generating the proper input script for
        both regular p2wkh output and p2wkh outputs nested within a regular p2sh
        output.

        Note that when using this method to sign inputs belonging to the wallet,
        the only items of the SignDescriptor that need to be populated are pkScript
        in the TxOut field, the value in that same field, and finally the input
        index.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SignMessage(self, request, context):
        """
        SignMessage signs a message with the key specified in the key locator. The
        returned signature is fixed-size LN wire format encoded.

        The main difference to SignMessage in the main RPC is that a specific key is
        used to sign the message instead of the node identity private key.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VerifyMessage(self, request, context):
        """
        VerifyMessage verifies a signature over a message using the public key
        provided. The signature must be fixed-size LN wire format encoded.

        The main difference to VerifyMessage in the main RPC is that the public key
        used to sign the message does not have to be a node known to the network.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeriveSharedKey(self, request, context):
        """
        DeriveSharedKey returns a shared secret key by performing Diffie-Hellman key
        derivation between the ephemeral public key in the request and the node's
        key specified in the key_desc parameter. Either a key locator or a raw
        public key is expected in the key_desc, if neither is supplied, defaults to
        the node's identity private key:
        P_shared = privKeyNode * ephemeralPubkey
        The resulting shared public key is serialized in the compressed format and
        hashed with sha256, resulting in the final key length of 256bit.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SignerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'SignOutputRaw': grpc.unary_unary_rpc_method_handler(
                    servicer.SignOutputRaw,
                    request_deserializer=signer__pb2.SignReq.FromString,
                    response_serializer=signer__pb2.SignResp.SerializeToString,
            ),
            'ComputeInputScript': grpc.unary_unary_rpc_method_handler(
                    servicer.ComputeInputScript,
                    request_deserializer=signer__pb2.SignReq.FromString,
                    response_serializer=signer__pb2.InputScriptResp.SerializeToString,
            ),
            'SignMessage': grpc.unary_unary_rpc_method_handler(
                    servicer.SignMessage,
                    request_deserializer=signer__pb2.SignMessageReq.FromString,
                    response_serializer=signer__pb2.SignMessageResp.SerializeToString,
            ),
            'VerifyMessage': grpc.unary_unary_rpc_method_handler(
                    servicer.VerifyMessage,
                    request_deserializer=signer__pb2.VerifyMessageReq.FromString,
                    response_serializer=signer__pb2.VerifyMessageResp.SerializeToString,
            ),
            'DeriveSharedKey': grpc.unary_unary_rpc_method_handler(
                    servicer.DeriveSharedKey,
                    request_deserializer=signer__pb2.SharedKeyRequest.FromString,
                    response_serializer=signer__pb2.SharedKeyResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'signrpc.Signer', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Signer(object):
    """Signer is a service that gives access to the signing functionality of the
    daemon's wallet.
    """

    @staticmethod
    def SignOutputRaw(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/signrpc.Signer/SignOutputRaw',
            signer__pb2.SignReq.SerializeToString,
            signer__pb2.SignResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ComputeInputScript(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/signrpc.Signer/ComputeInputScript',
            signer__pb2.SignReq.SerializeToString,
            signer__pb2.InputScriptResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SignMessage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/signrpc.Signer/SignMessage',
            signer__pb2.SignMessageReq.SerializeToString,
            signer__pb2.SignMessageResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VerifyMessage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/signrpc.Signer/VerifyMessage',
            signer__pb2.VerifyMessageReq.SerializeToString,
            signer__pb2.VerifyMessageResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeriveSharedKey(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/signrpc.Signer/DeriveSharedKey',
            signer__pb2.SharedKeyRequest.SerializeToString,
            signer__pb2.SharedKeyResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
